/**
 * Created by Samuel Gratzl on 25.01.2016.
 */
import { __extends } from "tslib";
import './style.scss';
import * as d3 from 'd3';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { rect } from 'phovea_core/src/geom';
import { mixin } from 'phovea_core/src';
import bindTooltip from 'phovea_d3/src/tooltip';
function createText(stats) {
    var r = '<table><tbody>';
    var keys = ['min', 'max', 'sum', 'mean', 'median', 'q1', 'q3', 'var', 'sd', 'n', 'nans', 'moment2', 'moment3', 'moment4', 'kurtosis', 'skewness'];
    keys.forEach(function (key) {
        var value = stats[key];
        r = r + "<tr><td>" + key + "</td><td>" + value + "</td></tr>";
    });
    r = r + "</tbody></table>";
    return r;
}
var BoxPlot = /** @class */ (function (_super) {
    __extends(BoxPlot, _super);
    function BoxPlot(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.options = {
            scale: [1, 1],
            rotate: 0
        };
        mixin(_this.options, options);
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(_this);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(BoxPlot.prototype, "rawSize", {
        get: function () {
            return [300, 50];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BoxPlot.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    BoxPlot.prototype.build = function ($parent) {
        var _this = this;
        var size = this.size, data = this.data;
        var $svg = $parent.append('svg').attr({
            width: size[0],
            height: size[1],
            'class': 'phovea-box'
        });
        var $t = $svg.append('g');
        var s = this.scale = d3.scale.linear().domain(this.data.desc.value.range).range([0, size[0]]).clamp(true);
        $t.append('path').attr({
            d: "M0,0 L0," + size[1] + " M0," + size[1] / 2 + " L" + size[0] + "," + size[1] / 2 + " M" + size[0] + ",0 L" + size[0] + "," + size[1],
            'class': 'axis'
        });
        data.statsAdvanced().then(function (stats) {
            var text = createText(stats);
            $t.append('rect').attr({
                x: s(stats.q1),
                y: '10%',
                width: s(stats.q3),
                height: '80%',
                'class': 'box'
            }).call(bindTooltip(text));
            $t.append('line').attr({
                x1: s(stats.median),
                x2: s(stats.median),
                y1: '10%',
                y2: '90%',
                'class': 'mean'
            });
            _this.markReady();
        });
        return $svg;
    };
    BoxPlot.prototype.locateImpl = function (range) {
        var that = this;
        if (range.isAll || range.isNone) {
            var r = this.scale.range();
            return Promise.resolve(rect(r[0], 0, r[1] - r[0], 50));
        }
        return this.data.data(range).then(function (data) {
            var ex = d3.extent(data, that.scale);
            return rect(ex[0], 0, ex[1] - ex[0], 50);
        });
    };
    return BoxPlot;
}(AVisInstance));
export { BoxPlot };
export default BoxPlot;
export function create(data, parent, options) {
    return new BoxPlot(data, parent, options);
}
//# sourceMappingURL=box.js.map