/**
 * Created by Samuel Gratzl on 26.01.2016.
 */
import { __extends } from "tslib";
import '../style.scss';
import * as d3 from 'd3';
import { onDOMNodeRemoved, mixin } from 'phovea_core/src';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { toSelectOperation } from 'phovea_core/src/idtype';
import bindTooltip from 'phovea_d3/src/tooltip';
import { createHistData, resolveHistMax } from './internal';
var Histogram = /** @class */ (function (_super) {
    __extends(Histogram, _super);
    function Histogram(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.options = {
            nbins: 5,
            total: true,
            duration: 200,
            scale: [1, 1],
            rotate: 0
        };
        mixin(_this.options, {
            nbins: Math.floor(Math.sqrt(data.length)),
        }, options);
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(_this);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(Histogram.prototype, "rawSize", {
        get: function () {
            return [200, 100];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Histogram.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    Histogram.prototype.build = function ($parent) {
        var _this = this;
        var size = this.size, data = this.data, o = this.options;
        var $svg = $parent.append('svg').attr({
            width: size[0],
            height: size[1],
            'class': 'phovea-histogram'
        });
        var $t = $svg.append('g');
        var $data = $t.append('g');
        var $highlight = $t.append('g').style('pointer-events', 'none').classed('phovea-select-selected', true);
        //using range bands with an ordinal scale for uniform distribution
        var xscale = this.xscale = d3.scale.ordinal().rangeBands([0, size[0]], 0.1);
        var yscale = this.yscale = d3.scale.linear().range([0, size[1]]);
        var l = function (event, type, selected) {
            if (!_this.histData) {
                return;
            }
            var highlights = _this.histData.map(function (entry, i) {
                var s = entry.range.intersect(selected);
                return {
                    i: i,
                    v: s.size()[0]
                };
            }).filter(function (entry) { return entry.v > 0; });
            var $m = $highlight.selectAll('rect').data(highlights);
            $m.enter().append('rect').attr('width', xscale.rangeBand());
            $m.attr({
                x: function (d) { return xscale(d.i); },
                y: function (d) { return yscale(yscale.domain()[1] - d.v); },
                height: 0
            });
            $m.transition().duration(o.duration).attr('height', function (d) {
                return yscale(d.v);
            });
            $m.exit().remove();
        };
        data.on('select', l);
        onDOMNodeRemoved($data.node(), function () {
            data.off('select', l);
        });
        var onClick = function (d) { return data.select(0, d.range, toSelectOperation(d3.event)); };
        this.data.hist(Math.floor(o.nbins)).then(function (hist) {
            _this.hist = hist;
            xscale.domain(d3.range(hist.bins));
            return resolveHistMax(hist, _this.options.total);
        }).then(function (histmax) {
            var hist = _this.hist;
            yscale.domain([0, histmax]);
            var histData = _this.histData = createHistData(hist, _this.data);
            var $m = $data.selectAll('rect').data(histData);
            $m.enter().append('rect')
                .attr('width', xscale.rangeBand())
                .call(bindTooltip(function (d) { return d.name + " " + d.v + " entries (" + Math.round(d.ratio * 100) + "%)"; }))
                .on('click', onClick);
            $m.attr({
                x: function (d, i) { return xscale(i); },
                fill: function (d) { return _this.options.color || d.color; },
                y: function (d) { return yscale(yscale.domain()[1] - d.v); },
                height: function (d) { return yscale(d.v); }
            });
            _this.markReady();
            data.selections().then(function (selected) {
                l(null, 'selected', selected);
            });
        });
        return $svg;
    };
    Histogram.prototype.locateImpl = function (range) {
        var _this = this;
        var size = this.rawSize;
        if (range.isAll || range.isNone) {
            return Promise.resolve({ x: 0, y: 0, w: size[0], h: size[1] });
        }
        return this.data.data(range).then(function (data) {
            var ex = d3.extent(data, function (value) { return _this.hist.binOf(value); });
            var h0 = _this.histData[ex[0]];
            var h1 = _this.histData[ex[1]];
            return Promise.resolve({
                x: _this.xscale(ex[0]),
                width: (_this.xscale(ex[1]) - _this.xscale(ex[0]) + _this.xscale.rangeBand()),
                height: _this.yscale(Math.max(h0.v, h1.v)),
                y: _this.yscale(_this.yscale.domain()[1] - Math.max(h0.v, h1.v))
            });
        });
    };
    Histogram.prototype.transform = function (scale, rotate) {
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        var size = this.rawSize;
        this.$node.attr({
            width: size[0] * scale[0],
            height: size[1] * scale[1]
        }).style('transform', 'rotate(' + rotate + 'deg)');
        this.$node.select('g').attr('transform', 'scale(' + scale[0] + ',' + scale[1] + ')');
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        return act;
    };
    return Histogram;
}(AVisInstance));
export default Histogram;
export function create(data, parent, options) {
    return new Histogram(data, parent, options);
}
//# sourceMappingURL=Histogram.js.map