/**
 * Created by Samuel Gratzl on 26.01.2016.
 */
import { __extends } from "tslib";
import '../style.scss';
import * as d3 from 'd3';
import { onDOMNodeRemoved, mixin } from 'phovea_core/src';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { toSelectOperation } from 'phovea_core/src/idtype';
import bindTooltip from 'phovea_d3/src/tooltip';
import { createHistData } from './internal';
var Mosaic = /** @class */ (function (_super) {
    __extends(Mosaic, _super);
    function Mosaic(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.options = {
            width: 20,
            initialScale: 10,
            duration: 200,
            heightTo: null,
            selectAble: true,
            scale: [1, 1],
            rotate: 0
        };
        mixin(_this.options, {
            scale: [1, _this.options.initialScale]
        }, options);
        if (typeof _this.options.heightTo === 'number') {
            _this.options.scale[1] = _this.options.heightTo / _this.data.dim[0];
        }
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(_this);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(Mosaic.prototype, "rawSize", {
        get: function () {
            return [this.options.width, this.data.dim[0]];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Mosaic.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    Mosaic.prototype.build = function ($parent) {
        var _this = this;
        var size = this.size, data = this.data, o = this.options;
        var $svg = $parent.append('svg').attr({
            width: size[0],
            height: size[1],
            'class': 'phovea-mosaic'
        });
        var $scale = $svg.append('g').attr('transform', 'scale(' + o.scale[0] + ',' + o.scale[1] + ')');
        var $data = $scale.append('g');
        var $highlight = $scale.append('g').style('pointer-events', 'none').classed('phovea-select-selected', true);
        var l = function (event, type, selected) {
            if (!_this.histData) {
                return;
            }
            var highlights = _this.histData.map(function (entry, i) {
                var s = entry.range.intersect(selected);
                return {
                    i: i,
                    acc: entry.acc,
                    v: s.size()[0]
                };
            }).filter(function (entry) { return entry.v > 0; });
            var $m = $highlight.selectAll('rect').data(highlights);
            $m.enter().append('rect').attr('width', '100%').classed('phovea-select-selected', true);
            $m.attr({
                y: function (d) { return d.acc; },
                height: function (d) { return d.v; }
            }).style('opacity', 0);
            $m.transition().duration(o.duration).style('opacity', 1);
            $m.exit().remove();
        };
        if (o.selectAble) {
            data.on('select', l);
            onDOMNodeRemoved($data.node(), function () { return data.off('select', l); });
        }
        var onClick = o.selectAble ? function (d) {
            data.select(0, d.range, toSelectOperation(d3.event));
        } : null;
        this.data.hist().then(function (hist) {
            _this.hist = hist;
            var histData = _this.histData = createHistData(hist, data);
            var $m = $data.selectAll('rect').data(histData);
            $m.enter().append('rect')
                .attr('width', '100%')
                .call(bindTooltip(function (d) { return d.name + " " + d.v + " entries (" + Math.round(d.ratio * 100) + "%)"; }))
                .on('click', onClick);
            $m.attr({
                y: function (d) { return d.acc; },
                height: function (d) { return d.v; },
                fill: function (d) { return d.color; }
            });
            _this.fire('built');
            _this.markReady();
            if (o.selectAble) {
                data.selections().then(function (selected) {
                    l(null, 'selected', selected);
                });
            }
        });
        return $svg;
    };
    Mosaic.prototype.locateImpl = function (range) {
        var _this = this;
        if (range.isAll || range.isNone) {
            var s = this.size;
            return Promise.resolve({ x: 0, y: 0, w: s[0], h: s[1] });
        }
        return this.data.data(range).then(function (data) {
            var ex = d3.extent(data, function (value) { return _this.hist.binOf(value); });
            var h0 = _this.histData[ex[0]];
            var h1 = _this.histData[ex[1]];
            var y = Math.min(h0.acc, h1.acc);
            var y2 = Math.max(h0.acc + h0.v, h1.acc + h1.v);
            var scale = _this.options.scale;
            return Promise.resolve({
                x: 0,
                width: _this.rawSize[0] * scale[0],
                height: (y2 - y) * scale[1],
                y: y * scale[1]
            });
        });
    };
    Mosaic.prototype.transform = function (scale, rotate) {
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        var size = this.rawSize;
        this.$node.attr({
            width: size[0] * scale[0],
            height: size[1] * scale[1]
        }).style('transform', 'rotate(' + rotate + 'deg)');
        this.$node.select('g').attr('transform', 'scale(' + scale[0] + ',' + scale[1] + ')');
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        return act;
    };
    return Mosaic;
}(AVisInstance));
export default Mosaic;
export function create(data, parent, options) {
    return new Mosaic(data, parent, options);
}
//# sourceMappingURL=Mosaic.js.map