/**
 * Created by Samuel Gratzl on 26.01.2016.
 */
import { __extends } from "tslib";
import '../style.scss';
import * as d3 from 'd3';
import { onDOMNodeRemoved, mixin } from 'phovea_core/src';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { toSelectOperation } from 'phovea_core/src/idtype';
import { vec2, polygon } from 'phovea_core/src/geom';
import bindTooltip from 'phovea_d3/src/tooltip';
import { resolveHistMax } from './internal';
function toPolygon(start, end, radius) {
    var r = [
        vec2(radius, radius),
        vec2(radius + Math.cos(start) * radius, radius + Math.sin(start) * radius),
        vec2(radius + Math.cos(end) * radius, radius + Math.sin(end) * radius)
    ];
    //approximate by triangle
    if (end - start > Math.PI) { //more than 180 degree use one more point
        r.splice(2, 0, vec2(radius + Math.cos((end - start) * 0.5) * radius, radius + Math.sin((end - start) * 0.5) * radius));
    }
    return polygon(r);
}
var Pie = /** @class */ (function (_super) {
    __extends(Pie, _super);
    function Pie(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.options = {
            radius: 50,
            innerRadius: 0,
            duration: 200,
            total: true,
            scale: [1, 1],
            rotate: 0
        };
        mixin(_this.options, options);
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(_this);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(Pie.prototype, "rawSize", {
        get: function () {
            var r = this.options.radius;
            return [r * 2, r * 2];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pie.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    Pie.prototype.build = function ($parent) {
        var _this = this;
        var size = this.size, data = this.data, o = this.options;
        var $svg = $parent.append('svg').attr({
            width: size[0],
            height: size[1],
            'class': 'phovea-pie'
        });
        var $base = $svg.append('g').attr('transform', 'translate(' + o.radius + ',' + o.radius + ')');
        var $data = $base.append('g');
        var $highlight = $base.append('g').style('pointer-events', 'none').classed('phovea-select-selected', true);
        var scale = this.scale = d3.scale.linear().range([0, 2 * Math.PI]);
        var arc = this.arc = d3.svg.arc().innerRadius(o.innerRadius).outerRadius(o.radius)
            .startAngle(function (d) { return scale(d.start); })
            .endAngle(function (d) { return scale(d.end); });
        var l = function (event, type, selected) {
            if (!_this.histData) {
                return;
            }
            var highlights = _this.histData.map(function (entry) {
                var s = entry.range.intersect(selected);
                return {
                    start: entry.start,
                    end: entry.start + s.size()[0]
                };
            }).filter(function (entry) { return entry.start < entry.end; });
            var $m = $highlight.selectAll('path').data(highlights);
            $m.enter().append('path');
            $m.exit().remove();
            $m.attr('d', arc);
        };
        data.on('select', l);
        onDOMNodeRemoved($data.node(), function () {
            data.off('select', l);
        });
        data.hist().then(function (hist) {
            _this.hist = hist;
            return resolveHistMax(hist, _this.options.total);
        }).then(function (total) {
            var hist = _this.hist;
            scale.domain([0, total]);
            var histData = _this.histData = [], cats = hist.categories;
            var prev = 0;
            var cols = hist.colors || d3.scale.category10().range();
            hist.forEach(function (b, i) {
                histData[i] = {
                    name: (typeof cats[i] === 'string') ? cats[i] : cats[i].name,
                    start: prev,
                    size: b,
                    ratio: b / total,
                    end: prev + b,
                    color: (cats[i].color === undefined) ? cols[i] : cats[i].color,
                    range: hist.range(i)
                };
                prev += b;
            });
            var $m = $data.selectAll('path').data(histData);
            $m.enter()
                .append('path')
                .call(bindTooltip(function (d) { return d.name + ' ' + (d.size) + ' entries (' + Math.round(d.ratio * 100) + '%)'; }))
                .on('click', function (d) { return data.select(0, d.range, toSelectOperation(d3.event)); });
            $m.attr('d', arc)
                .attr('fill', function (d) { return d.color; })
                .style('opacity', 0);
            //fade in animation
            $m.transition()
                .duration(o.duration)
                .delay(function (d, i) { return i * o.duration; })
                .style('opacity', 1);
            _this.markReady();
            data.selections().then(function (selected) { return l(null, 'selected', selected); });
        });
        return $svg;
    };
    Pie.prototype.locateImpl = function (range) {
        var _this = this;
        var o = this.options;
        if (range.isAll || range.isNone) {
            return Promise.resolve({ x: o.radius, y: o.radius, radius: o.radius });
        }
        return this.data.data(range).then(function (data) {
            var ex = d3.extent(data, function (value) { return _this.hist.binOf(value); });
            var startAngle = _this.scale(_this.histData[ex[0]].start);
            var endAngle = _this.scale(_this.histData[ex[1]].end);
            return Promise.resolve(toPolygon(startAngle, endAngle, o.radius));
        });
    };
    Pie.prototype.transform = function (scale, rotate) {
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        this.$node.attr({
            width: this.options.radius * 2 * scale[0],
            height: this.options.radius * 2 * scale[1]
        }).style('transform', 'rotate(' + rotate + 'deg)');
        this.$node.select('g').attr('transform', 'scale(' + scale[0] + ',' + scale[1] + ')translate(' + this.options.radius + ',' + this.options.radius + ')');
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        return act;
    };
    return Pie;
}(AVisInstance));
export default Pie;
export function create(data, parent, options) {
    return new Pie(data, parent, options);
}
//# sourceMappingURL=Pie.js.map