/**
 * Created by Samuel Gratzl on 26.12.2016.
 */
import * as d3 from 'd3';
import { VALUE_TYPE_CATEGORICAL } from 'phovea_core/src/datatype';
function createCategoricalHistData(hist) {
    var categories = hist.categories;
    var cols = hist.colors || d3.scale.category10().range();
    var total = hist.validCount;
    var data = [];
    var acc = 0;
    hist.forEach(function (b, i) {
        data[i] = {
            v: b,
            acc: acc,
            ratio: b / total,
            range: hist.range(i),
            name: (typeof categories[i] === 'string') ? categories[i] : categories[i].name,
            color: (categories[i].color === undefined) ? cols[i] : categories[i].color
        };
        acc += b;
    });
    return data;
}
function createNumericalHistData(hist, range) {
    var data = [], cols = d3.scale.linear().domain(range).range(['#111111', '#999999']), total = hist.validCount, binWidth = (range[1] - range[0]) / hist.bins;
    var acc = 0;
    hist.forEach(function (b, i) {
        data[i] = {
            v: b,
            acc: acc,
            ratio: b / total,
            range: hist.range(i),
            name: 'Bin ' + (i + 1) + ' (center: ' + d3.round((i + 0.5) * binWidth, 2) + ')',
            color: cols((i + 0.5) * binWidth)
        };
        acc += b;
    });
    return data;
}
export function createHistData(hist, data) {
    if (data.desc.type === 'stratification') {
        return createCategoricalHistData(hist);
    }
    var d = data.valuetype;
    if (d.type === VALUE_TYPE_CATEGORICAL) {
        return createCategoricalHistData(hist);
    }
    return createNumericalHistData(hist, d.range);
}
export function resolveHistMax(hist, totalHeight) {
    var op = typeof totalHeight === 'function' ? totalHeight : function () { return totalHeight; };
    return Promise.resolve(op(hist)).then(function (r) {
        if (r === true) {
            return hist.validCount;
        }
        if (r === false) {
            return hist.largestBin;
        }
        return r;
    });
}
//# sourceMappingURL=internal.js.map