/**
 * Created by Samuel Gratzl on 05.08.2014.
 */
import { __extends } from "tslib";
import './style.scss';
import * as d3 from 'd3';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { mixin } from 'phovea_core/src';
var ForceDirectedGraphVis = /** @class */ (function (_super) {
    __extends(ForceDirectedGraphVis, _super);
    function ForceDirectedGraphVis(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.parent = parent;
        _this.options = {
            scale: [1, 1],
            rotate: 0,
            colors: true
        };
        mixin(_this.options, options);
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(data);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(ForceDirectedGraphVis.prototype, "rawSize", {
        /**
         * the raw size without any scaling factors applied
         * @returns {any[]}
         */
        get: function () {
            return [300, 300];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ForceDirectedGraphVis.prototype, "node", {
        /**
         * access to the HTML Element of this visualization
         * @returns {Element}
         */
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * get/set an option of this vis
     * @param name
     * @param val
     * @returns {any}
     */
    ForceDirectedGraphVis.prototype.option = function (name, val) {
        if (arguments.length === 1) {
            return this.options[name];
        }
        else {
            this.fire('option', name, val, this.options[name]);
            this.fire('option.' + name, val, this.options[name]);
            this.options[name] = val;
            //handle option change
        }
    };
    /**
     * transform this visualization given the scaling and rotation factor
     * @param scale a two number array
     * @param rotate a factor in degree
     * @returns {any}
     */
    ForceDirectedGraphVis.prototype.transform = function (scale, rotate) {
        if (rotate === void 0) { rotate = 0; }
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        var raw = this.rawSize;
        this.$node.style('transform', 'rotate(' + rotate + 'deg)');
        this.$node.attr('width', raw[0] * scale[0]).attr('height', raw[1] * scale[1]);
        this.$node.select('g').attr('transform', 'scale(' + scale[0] + ',' + scale[1] + ')');
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        return act;
    };
    ForceDirectedGraphVis.prototype.build = function ($parent) {
        var _this = this;
        //create a svg and append a group for the transform
        var scaleFactor = this.options.scale, size = this.size;
        //create the svg
        var $svg = $parent.append('svg').attr({
            width: size[0],
            height: size[1],
            'class': 'phovea-graph-force'
        });
        var $root = $svg.append('g').attr('transform', 'scale(' + scaleFactor[0] + ',' + scaleFactor[1] + ')');
        var colors = d3.scale.category10().range().slice();
        this.data.impl().then(function (graph) {
            var nodes = graph.nodes.map(function (n) { return ({ v: n }); });
            var lookup = d3.map(nodes, function (d) { return String(d.v.id); });
            var edges = graph.edges.map(function (n) { return ({
                v: n,
                source: lookup.get(String(n.source.id)),
                target: lookup.get(String(n.target.id))
            }); });
            var f = d3.layout.force()
                .size(_this.rawSize);
            f.nodes(nodes).links(edges);
            var $links = $root.selectAll('.edge').data(edges);
            $links.enter().append('line').classed('edge', true);
            $links.attr('data-type', function (d) { return d.v.type; });
            $links.exit().remove();
            var $nodes = $root.selectAll('.node').data(nodes);
            $nodes.enter().append('circle')
                .classed('node', true).attr('r', 5)
                .call(f.drag)
                .on('click', function (d) {
                console.log(d.v);
            })
                .append('title');
            $nodes.attr('data-type', function (d) { return d.v.type; });
            if (_this.options.colors) {
                $links.style('stroke', function (d) {
                    var c = lookup[d.v.type];
                    if (!c) {
                        c = colors.shift() || 'gray';
                        lookup[d.v.type] = c;
                    }
                    return c;
                });
                $nodes.style('fill', function (d) {
                    var c = lookup[d.v.type];
                    if (!c) {
                        c = colors.shift() || 'gray';
                        lookup[d.v.type] = c;
                    }
                    return c;
                });
            }
            $nodes.select('title').text(function (d) { return d.v.id + ' ' + d.v.type; });
            f.on('tick', function () {
                $links.attr({
                    x1: function (d) { return d.source.x; },
                    y1: function (d) { return d.source.y; },
                    x2: function (d) { return d.target.x; },
                    y2: function (d) { return d.target.y; }
                });
                $nodes.attr({
                    cx: function (d) { return d.x; },
                    cy: function (d) { return d.y; }
                });
            });
            _this.markReady();
            f.start();
        });
        return $svg;
    };
    return ForceDirectedGraphVis;
}(AVisInstance));
export { ForceDirectedGraphVis };
export function create(data, parent, options) {
    return new ForceDirectedGraphVis(data, parent, options);
}
//# sourceMappingURL=force_directed_graph.js.map