/**
 * Created by Samuel Gratzl on 26.12.2016.
 */
import { __read, __spread } from "tslib";
import * as d3 from 'd3';
import { cell } from 'phovea_core/src/range';
import { onDOMNodeRemoved } from 'phovea_core/src';
import { toSelectOperation, defaultSelectionType } from 'phovea_core/src/idtype';
import { ESelectOption } from './IHeatMapRenderer';
var AHeatMapCanvasRenderer = /** @class */ (function () {
    function AHeatMapCanvasRenderer(selectAble, options) {
        if (selectAble === void 0) { selectAble = ESelectOption.CELL; }
        this.selectAble = selectAble;
        this.options = options;
    }
    AHeatMapCanvasRenderer.prototype.rescale = function ($node, dim, scale) {
        var _this = this;
        $node.selectAll('canvas.phovea-heatmap-selection').attr({
            width: dim[1] * scale[0],
            height: dim[0] * scale[1]
        });
        if (this.selectAble !== ESelectOption.NONE) {
            $node.datum().productSelections().then(function (selected) {
                _this.redrawSelection($node.select('canvas.phovea-heatmap-selection').node(), dim, defaultSelectionType, selected);
            });
        }
    };
    AHeatMapCanvasRenderer.prototype.redrawSelection = function (canvas, dim, type, selected) {
        var ctx = canvas.getContext('2d');
        ctx.save();
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        ctx.fillStyle = 'orange';
        ctx.strokeStyle = 'orange';
        if (selected.length === 0) {
            ctx.restore();
            return;
        }
        if (selected.some(function (a) { return a.isAll; })) {
            ctx.fillRect(0, 0, canvas.width, canvas.height);
            ctx.restore();
            return;
        }
        if (this.options.mode === 'sm') {
            ctx.scale(canvas.width / dim[1], canvas.height / dim[0]);
            selected.forEach(function (cell) {
                cell.product(function (indices) {
                    var _a = __read(indices, 2), i = _a[0], j = _a[1];
                    ctx.fillRect(j, i, 1, 1);
                }, dim);
            });
        }
        else {
            var cw_1 = canvas.width / dim[1];
            var ch_1 = canvas.height / dim[0];
            selected.forEach(function (cell) {
                cell.product(function (indices) {
                    var _a = __read(indices, 2), i = _a[0], j = _a[1];
                    ctx.strokeRect(j * cw_1, i * ch_1, cw_1, ch_1);
                }, dim);
            });
        }
        ctx.restore();
    };
    AHeatMapCanvasRenderer.prototype.buildSelection = function (data, $root, scale) {
        var _this = this;
        if (this.selectAble === ESelectOption.NONE) {
            return;
        }
        var dims = data.dim;
        var width = dims[1], height = dims[0];
        var $selection = $root.append('canvas').attr({
            width: width * scale[0],
            height: height * scale[1],
            'class': 'phovea-heatmap-selection'
        });
        var toCoord = function (evt) {
            var c = $selection.node(), rect = c.getBoundingClientRect();
            var x = evt.clientX - rect.left, y = evt.clientY - rect.top;
            var i = Math.floor(width * x / c.width), j = Math.floor(height * y / c.height);
            return [j, i];
        };
        $selection.on('click', function () {
            var ij = toCoord(d3.event);
            data.selectProduct([cell.apply(void 0, __spread(ij))], toSelectOperation(d3.event));
        });
        var l = function (event, type, selected) {
            _this.redrawSelection($selection.node(), dims, type, selected);
        };
        data.on('selectProduct', l);
        onDOMNodeRemoved($selection.node(), function () {
            data.off('selectProduct', l);
        });
        data.productSelections().then(function (selected) {
            _this.redrawSelection($selection.node(), dims, 'selected', selected);
        });
    };
    return AHeatMapCanvasRenderer;
}());
export { AHeatMapCanvasRenderer };
export default AHeatMapCanvasRenderer;
//# sourceMappingURL=AHeatMapCanvasRenderer.js.map