/**
 * Created by Samuel Gratzl on 26.12.2016.
 */
import { __extends } from "tslib";
import '../style.scss';
import * as d3 from 'd3';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { rect } from 'phovea_core/src/geom';
import { mixin } from 'phovea_core/src';
import { selectionUtil } from 'phovea_d3/src/d3util';
import { defaultColor, defaultDomain, toScale, isMissing } from './internal';
var HeatMap1D = /** @class */ (function (_super) {
    __extends(HeatMap1D, _super);
    function HeatMap1D(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.parent = parent;
        _this.options = {
            initialScale: 10,
            width: 20,
            heightTo: null,
            scale: [1, 1],
            rotate: 0,
            missingColor: '#d400c2'
        };
        var value = _this.data.valuetype;
        mixin(_this.options, {
            color: defaultColor(value),
            domain: defaultDomain(value)
        }, options);
        _this.options.scale = [1, _this.options.initialScale];
        if (_this.options.heightTo) {
            _this.options.scale[1] = _this.options.heightTo / _this.data.dim[0];
        }
        _this.colorer = toScale(value).domain(_this.options.domain).range(_this.options.color);
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(data);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(HeatMap1D.prototype, "rawSize", {
        get: function () {
            var d = this.data.dim;
            return [this.options.width, d[0]];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HeatMap1D.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    HeatMap1D.prototype.option = function (name, val) {
        if (arguments.length === 1) {
            return this.options[name];
        }
        else {
            this.fire('option', name, val, this.options[name]);
            this.fire('option.' + name, val, this.options[name]);
            this.options[name] = val;
            switch (name) {
                case 'color':
                case 'domain':
                    this.recolor();
                    break;
            }
        }
    };
    HeatMap1D.prototype.locateImpl = function (range) {
        var dims = this.data.dim;
        var height = dims[0];
        function l(r, max, s) {
            if (r.isAll || r.isNone) {
                return [0, max * s];
            }
            var ex = d3.extent(r.asList());
            return [ex[0] * s, (ex[1] - ex[0] + 1) * s];
        }
        var yh = l(range.dim(0), height, this.options.scale[1]);
        return Promise.resolve(rect(0, yh[0], 20, yh[1]));
    };
    HeatMap1D.prototype.transform = function (scale, rotate) {
        if (rotate === void 0) { rotate = 0; }
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        var dims = this.data.dim;
        var width = this.options.width, height = dims[0];
        this.$node.attr({
            width: width * scale[0],
            height: height * scale[1]
        }).style('transform', 'rotate(' + rotate + 'deg)');
        this.$node.select('g').attr('transform', 'scale(' + scale[0] + ',' + scale[1] + ')');
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        return act;
    };
    HeatMap1D.prototype.recolor = function () {
        var _this = this;
        var c = this.colorer;
        c.domain(this.options.domain).range(this.options.color);
        this.$node.selectAll('rect').attr('fill', function (d) { return isMissing(d) ? _this.options.missingColor : c(d); });
    };
    HeatMap1D.prototype.build = function ($parent) {
        var _this = this;
        var dims = this.data.dim;
        var width = this.options.width, height = dims[0];
        var $svg = $parent.append('svg').attr({
            width: width,
            height: height * this.options.initialScale,
            'class': 'phovea-heatmap'
        });
        var $g = $svg.append('g').attr('transform', 'scale(1,' + this.options.initialScale + ')');
        var c = this.colorer;
        var t = this.data.data();
        t.then(function (arr) {
            var $rows = $g.selectAll('rect').data(arr);
            var onClick = selectionUtil(_this.data, $g, 'rect');
            $rows.enter().append('rect').on('click', onClick).attr({
                width: _this.options.width,
                height: 1
            }).append('title').text(String);
            $rows.attr({
                fill: function (d) { return isMissing(d) ? _this.options.missingColor : c(d); },
                y: function (d, i) { return i; }
            });
            $rows.exit().remove();
            _this.markReady();
        });
        return $svg;
    };
    return HeatMap1D;
}(AVisInstance));
export default HeatMap1D;
export function create(data, parent, options) {
    return new HeatMap1D(data, parent, options);
}
//# sourceMappingURL=HeatMap1D.js.map