/**
 * Created by Samuel Gratzl on 26.12.2016.
 */
import { __extends } from "tslib";
import * as d3 from 'd3';
import { isMissing } from './internal';
import { ESelectOption } from './IHeatMapRenderer';
import AHeatMapCanvasRenderer from './AHeatMapCanvasRenderer';
var HeatMapCanvasRenderer = /** @class */ (function (_super) {
    __extends(HeatMapCanvasRenderer, _super);
    function HeatMapCanvasRenderer(selectAble, options) {
        if (selectAble === void 0) { selectAble = ESelectOption.CELL; }
        var _this = _super.call(this, selectAble, options) || this;
        _this.ready = false;
        return _this;
    }
    HeatMapCanvasRenderer.prototype.rescale = function ($node, dim, scale) {
        $node.selectAll('canvas').attr({
            width: dim[1] * scale[0],
            height: dim[0] * scale[1]
        });
        if (this.ready) {
            this.redrawImpl(this.imageData, $node, scale);
        }
        _super.prototype.rescale.call(this, $node, dim, scale);
    };
    HeatMapCanvasRenderer.prototype.redraw = function ($node, scale) {
        if (this.ready) {
            this.redrawImpl(this.imageData, $node, scale);
        }
    };
    HeatMapCanvasRenderer.prototype.recolor = function ($node, data, color, scale) {
        var _this = this;
        var rgba = this.imageData.data;
        if (this.ready) {
            data.data().then(function (arr) {
                _this.genImage(rgba, arr, data.ncol, color);
                _this.redrawImpl(_this.imageData, $node, scale);
            });
        }
    };
    HeatMapCanvasRenderer.prototype.genImage = function (rgba, arr, ncol, c) {
        var _this = this;
        arr.forEach(function (row, j) {
            var t = j * ncol;
            row.forEach(function (cell, i) {
                var color = d3.rgb(isMissing(cell) ? _this.options.missingColor : c(cell));
                rgba[(t + i) * 4] = color.r;
                rgba[(t + i) * 4 + 1] = color.g;
                rgba[(t + i) * 4 + 2] = color.b;
                rgba[(t + i) * 4 + 3] = 255;
            });
        });
    };
    HeatMapCanvasRenderer.prototype.redrawImpl = function (imageData, $root, scale) {
        var context = $root.select('canvas').node().getContext('2d');
        context.msImageSmoothingEnabled = false;
        //if (context.hasOwnProperty('imageSmoothingEnabled')) {
        context.imageSmoothingEnabled = false;
        //}
        if (scale[0] === 1 && scale[1] === 1) {
            //don't nee
            context.putImageData(imageData, 0, 0);
        }
        else {
            var tmp = $root.node().ownerDocument.createElement('canvas');
            tmp.width = imageData.width;
            tmp.height = imageData.height;
            var d = tmp.getContext('2d');
            d.putImageData(imageData, 0, 0);
            context.save();
            context.scale(scale[0], scale[1]);
            context.drawImage(tmp, 0, 0);
            context.restore();
            d3.select(tmp).remove();
        }
    };
    HeatMapCanvasRenderer.prototype.build = function (data, $parent, scale, c, onReady) {
        var _this = this;
        var dims = data.dim;
        var width = dims[1], height = dims[0];
        var $root = $parent.append('div').attr('class', 'phovea-heatmap');
        var $canvas = $root.append('canvas').attr({
            width: width * scale[0],
            height: height * scale[1],
            'class': 'phovea-heatmap-data'
        });
        this.imageData = $canvas.node().getContext('2d').createImageData(width, height); //new (<any>ImageData)(data.ncol, data.nrow);
        var rgba = this.imageData.data;
        data.data().then(function (arr) {
            _this.ready = true;
            _this.genImage(rgba, arr, data.ncol, c);
            onReady();
        });
        _super.prototype.buildSelection.call(this, data, $root, scale);
        return $root;
    };
    return HeatMapCanvasRenderer;
}(AHeatMapCanvasRenderer));
export default HeatMapCanvasRenderer;
//# sourceMappingURL=HeatMapCanvasRenderer.js.map