/**
 * Created by Samuel Gratzl on 26.12.2016.
 */
import { __extends } from "tslib";
import * as d3 from 'd3';
import { all } from 'phovea_core/src/range';
import { ESelectOption } from './IHeatMapRenderer';
import AHeatMapCanvasRenderer from './AHeatMapCanvasRenderer';
import { sendAPI, encodeParams, MAX_URL_LENGTH } from 'phovea_core/src/ajax';
import parseRange from 'phovea_core/src/range/parser';
import { prepareHeatmapUrlParameter } from 'phovea_core/src/matrix/loader';
function ensureHex(color) {
    var rgb = d3.rgb(color);
    var toHex = function (d) { return ('00' + d.toString(16)).slice(-2); };
    return "#" + toHex(rgb.r) + toHex(rgb.g) + toHex(rgb.b);
}
var HeatMapImageRenderer = /** @class */ (function (_super) {
    __extends(HeatMapImageRenderer, _super);
    function HeatMapImageRenderer(selectAble, options) {
        if (selectAble === void 0) { selectAble = ESelectOption.CELL; }
        var _this = _super.call(this, selectAble, options) || this;
        _this.ready = false;
        return _this;
    }
    HeatMapImageRenderer.prototype.rescale = function ($node, dim, scale) {
        $node.selectAll('canvas').attr({
            width: dim[1] * scale[0],
            height: dim[0] * scale[1]
        });
        if (this.ready) {
            this.redrawImpl($node, scale);
        }
        _super.prototype.rescale.call(this, $node, dim, scale);
    };
    HeatMapImageRenderer.prototype.redraw = function ($node, scale) {
        if (this.ready) {
            this.redrawImpl($node, scale);
        }
    };
    HeatMapImageRenderer.prototype.redrawImpl = function ($root, scale) {
        var canvas = $root.select('canvas').node();
        var ctx = canvas.getContext('2d');
        ctx.msImageSmoothingEnabled = false;
        //if (context.hasOwnProperty('imageSmoothingEnabled')) {
        ctx.imageSmoothingEnabled = false;
        //}
        if (scale[0] === 1 && scale[1] === 1) {
            ctx.drawImage(this.image, 0, 0);
        }
        else {
            ctx.save();
            ctx.scale(scale[0], scale[1]);
            //console.log('draw with scale', scale, this.image.width, this.image.height);
            ctx.drawImage(this.image, 0, 0);
            ctx.restore();
        }
        //apply color scale
        /*if (false) { //FIXME
         let imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
         let data = imageData.data;
         var help  = d3.scale.linear().domain([0,255]).range(this.color.domain());
         for (let i = 0; i < data.length; i += 4) {
         //gray scale
         let v = data[i];
         //to convert to domain value or use a different scale
         var color = d3.rgb(this.color(help(v)));
         data[i] = color.r;
         data[i + 1] = color.g;
         data[i + 2] = color.b;
         data[i + 3] = 255;
         }
         ctx.putImageData(imageData, 0, 0);
         }*/
    };
    HeatMapImageRenderer.prototype.recolor = function ($node, data, color, scale) {
        //can't do that
        this.color = color;
        this.redrawImpl($node, scale);
    };
    HeatMapImageRenderer.prototype.build = function (data, $parent, scale, c, onReady) {
        var _this = this;
        this.color = c;
        var dims = data.dim;
        var width = dims[1], height = dims[0];
        var $root = $parent.append('div').attr('class', 'phovea-heatmap');
        $root.append('canvas').attr({
            width: width * scale[0],
            height: height * scale[1],
            'class': 'phovea-heatmap-data'
        });
        this.image = new Image();
        this.image.onload = function () {
            _this.ready = true;
            onReady();
        };
        var domain = c.domain();
        var args = {
            range: [domain[0], domain[domain.length - 1]],
            missing: ensureHex(this.options.missingColor)
        };
        function arrEqual(a, b) {
            if (a.length !== b.length) {
                return false;
            }
            return a.every(function (ai, i) { return ai === b[i]; });
        }
        var colors = c.range();
        if (arrEqual(colors, ['black', 'white'])) {
            //default scale
        }
        else if (arrEqual(colors, ['white', 'red'])) {
            args.palette = 'white_red';
        }
        else if (arrEqual(colors, ['blue', 'white', 'red'])) {
            args.palette = 'blue_white_red';
        }
        else if (colors.length === 2 || colors.length === 3) {
            args.palette = colors.map(ensureHex).join('-');
        }
        // persist to get range and create range object again
        // TODO: make range property on matrix public
        var range = parseRange(data.persist().range);
        var params = prepareHeatmapUrlParameter(range, args);
        var url = "/dataset/matrix/" + data.desc.id + "/data";
        var encoded = encodeParams(params);
        if (encoded && (url.length + encoded.length >= MAX_URL_LENGTH)) {
            // use post instead
            sendAPI(url, params, 'POST', 'blob').then(function (image) {
                var imageURL = window.URL.createObjectURL(image);
                _this.image.src = imageURL;
            });
        }
        else {
            this.image.src = data.heatmapUrl(all(), args);
        }
        _super.prototype.buildSelection.call(this, data, $root, scale);
        return $root;
    };
    return HeatMapImageRenderer;
}(AHeatMapCanvasRenderer));
export default HeatMapImageRenderer;
//# sourceMappingURL=HeatMapImageRenderer.js.map