/**
 * Created by Samuel Gratzl on 01.10.2015.
 */
import { __extends } from "tslib";
import './style.scss';
import * as d3 from 'd3';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { mixin } from 'phovea_core/src';
var KaplanMeierPlot = /** @class */ (function (_super) {
    __extends(KaplanMeierPlot, _super);
    function KaplanMeierPlot(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.parent = parent;
        _this.options = {
            scale: [1, 1],
            rotate: 0,
            width: 300,
            height: 300,
            maxTime: function (died) { return died[died.length - 1]; }
        };
        _this.line = d3.svg.line().interpolate('step');
        //var value = (<any>this.data.desc).value;
        mixin(_this.options, options);
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(data);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(KaplanMeierPlot.prototype, "rawSize", {
        get: function () {
            return [this.options.width, this.options.height];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KaplanMeierPlot.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    KaplanMeierPlot.prototype.locateImpl = function (range) {
        //TODO
        return Promise.resolve(null);
    };
    KaplanMeierPlot.prototype.transform = function (scale, rotate) {
        if (rotate === void 0) { rotate = 0; }
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        var width = this.options.width, height = this.options.height;
        this.$node.attr({
            width: width * scale[0],
            height: height * scale[1]
        }).style('transform', 'rotate(' + rotate + 'deg)');
        this.$node.select('g').attr('transform', 'scale(' + scale[0] + ',' + scale[1] + ')');
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        return act;
    };
    KaplanMeierPlot.prototype.build = function ($parent) {
        var _this = this;
        var width = this.options.width, height = this.options.height, scale = this.options.scale || [1, 1];
        var $svg = $parent.append('svg').attr({
            width: width * scale[0],
            height: height * scale[1],
            'class': 'phovea-kaplanmeier'
        });
        var $g = $svg.append('g');
        var xscale = d3.scale.linear().range([0, width]);
        var yscale = d3.scale.linear().range([0, height]);
        this.line
            .x(function (d) { return xscale(d[0]); })
            .y(function (d) { return yscale(d[1]); });
        this.data.data().then(function (arr) {
            //TODO
            var died = arr.filter(function (a) { return !isNaN(a) && a !== null; }).map(function (a) { return Math.abs(a); });
            died.sort(d3.ascending);
            //const alive = arr.length - died.length;
            yscale.domain([0, arr.length]);
            Promise.resolve(_this.options.maxTime(died)).then(function (maxAxisTime) {
                xscale.domain([0, maxAxisTime]);
                //0 ... 100%
                var points = [[0, 0]];
                var prevI = 0;
                for (var i = 1; i < died.length; ++i) {
                    while (died[i] === died[i - 1] && i < died.length) {
                        ++i;
                    }
                    points.push([died[prevI], prevI + 1]);
                    prevI = i;
                }
                if (died.length > 0) {
                    points.push([died[prevI], prevI + 1]);
                }
                points.push([maxAxisTime, died.length]);
                $g.append('path').datum(points).attr('d', _this.line);
                _this.markReady();
            });
        });
        return $svg;
    };
    return KaplanMeierPlot;
}(AVisInstance));
export { KaplanMeierPlot };
export function create(data, parent, options) {
    return new KaplanMeierPlot(data, parent, options);
}
//# sourceMappingURL=kaplanmeier.js.map