/**
 * Created by Samuel Gratzl on 25.01.2016.
 */
import { __extends } from "tslib";
import '../../style.scss';
import { select, extent, selection } from 'd3';
import { mixin } from 'phovea_core/src';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { selectionUtil } from 'phovea_d3/src/d3util';
import { rect } from 'phovea_core/src/geom';
var DEFAULT_OPTIONS = {
    scale: [1, 1],
    rotate: 0,
    width: 200,
    rowHeight: 20,
    cssClass: ''
};
var AList = /** @class */ (function (_super) {
    __extends(AList, _super);
    function AList(data, parent, options) {
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.parent = parent;
        _this.options = mixin({}, DEFAULT_OPTIONS, options);
        _this.$node = select(parent).append('div').attr('class', 'phovea-list ' + _this.options.cssClass);
        _this.$node.datum(_this);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(AList.prototype, "rawSize", {
        get: function () {
            return [this.options.width, this.data.length * this.options.rowHeight];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AList.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    AList.prototype.locateImpl = function (range) {
        var w = this.node.clientWidth;
        var $node = select(this.node);
        var a, b;
        if (range.isAll || range.isNone) {
            a = this.node.firstElementChild;
            b = this.node.lastElementChild;
        }
        else {
            var ex = extent(range.dim(0).iter().asList());
            a = this.node.childNodes.item(ex[0]);
            b = this.node.childNodes.item(ex[1]);
        }
        return Promise.resolve(rect(0, a.offsetTop, w, b.offsetTop + b.clientHeight - a.offsetTop));
    };
    AList.prototype.transform = function (scale, rotate) {
        if (rotate === void 0) { rotate = 0; }
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        this.$node.style('transform', 'rotate(' + rotate + 'deg)');
        this.$node.style('width', scale[0] * this.options.width + "px");
        this.$node.style('height', scale[1] * this.data.length * this.options.rowHeight + "px");
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        this.update();
        return act;
    };
    AList.prototype.update = function () {
        this.render(selection(), this.$node.selectAll('div'));
    };
    AList.prototype.build = function () {
        var _this = this;
        var scale = this.options.scale;
        this.$node.style('width', scale[0] * this.options.width + "px");
        this.$node.style('height', scale[1] * this.data.length * this.options.rowHeight + "px");
        var onClick = selectionUtil(this.data, this.$node, 'div');
        this.data.data().then(function (arr) {
            var $rows = _this.$node.selectAll('div').data(arr);
            var $rowsEnter = $rows.enter().append('div').on('click', onClick);
            _this.render($rowsEnter, $rows);
            $rows.exit().remove();
            _this.markReady();
        });
    };
    return AList;
}(AVisInstance));
export { AList };
export default AList;
//# sourceMappingURL=AList.js.map