/**
 * Created by Samuel Gratzl on 05.08.2014.
 */
import { __extends } from "tslib";
import './style.scss';
import * as d3 from 'd3';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { rect } from 'phovea_core/src/geom';
import { selectionUtil } from 'phovea_d3/src/d3util';
import { mixin } from 'phovea_core/src';
var Table = /** @class */ (function (_super) {
    __extends(Table, _super);
    function Table(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.options = {
            scale: [1, 1],
            rotate: 0
        };
        mixin(_this.options, options);
        var $p = d3.select(parent);
        switch (data.desc.type) { //depending on the type of the data, create a different table
            case 'matrix':
                var dmatrix = data;
                _this.$node = _this.build($p, [dmatrix.cols(), dmatrix.rows(), dmatrix.data()]);
                break;
            case 'table':
                var dtable = data;
                _this.$node = _this.build($p, [dtable.cols().map(function (v) { return v.desc.name; }), dtable.rows(), _this.data.data()]);
                break;
            case 'vector':
                var dvector = data;
                _this.$node = _this.build($p, [
                    [dvector.desc.name],
                    dvector.names(),
                    dvector.data().then(function (data) { return data.map(function (d) { return [d]; }); })
                ]);
                break;
        }
        _this.$node.datum(data);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(Table.prototype, "rawSize", {
        get: function () {
            var dim = this.data.dim;
            return [(dim[1] || 1) * 200, dim[0] * 22];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Table.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    Table.prototype.locateImpl = function (range) {
        var $tbody = d3.select(this.node).select('tbody');
        var offset = $tbody.node().offsetTop, w = $tbody.node().clientWidth;
        var a, b;
        if (range.isAll || range.isNone) {
            b = $tbody.select('tr:last').node();
            return Promise.resolve(rect(0, offset, w, b.offsetTop + b.clientHeight));
        }
        var ex = d3.extent(range.dim(0).iter().asList());
        a = $tbody.select('tr:nth-child(' + (ex[0] + 1) + ')').node();
        b = $tbody.select('tr:nth-child(' + (ex[1] + 1) + ')').node();
        return Promise.resolve(rect(0, a.offsetTop, w, b.offsetTop + b.clientHeight - a.offsetTop));
    };
    Table.prototype.transform = function (scale, rotate) {
        if (rotate === void 0) { rotate = 0; }
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        this.$node.style('transform', 'rotate(' + rotate + 'deg)scale(' + scale[0] + ',' + scale[1] + ')');
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        return act;
    };
    Table.prototype.build = function ($parent, promises) {
        var _this = this;
        var $table = $parent.append('table').attr('class', 'phovea-table');
        $table.append('thead').append('tr');
        $table.append('tbody');
        var onClick = selectionUtil(this.data, $table.select('tbody'), 'tr');
        Promise.all(promises).then(function (arr) {
            var cols = arr[0], rows = arr[1], d = arr[2];
            var $headers = $table.select('thead tr').selectAll('th').data(['ID'].concat(cols));
            $headers.enter().append('th');
            $headers.text(String);
            $headers.exit().remove();
            var $rows = $table.select('tbody').selectAll('tr').data(d);
            $rows.enter().append('tr').on('click', onClick);
            $rows.each(function (row, i) {
                var $header = d3.select(this).selectAll('th').data(rows.slice(i, i + 1));
                $header.enter().append('th');
                $header.text(String);
                $header.exit().remove();
                var $row = d3.select(this).selectAll('td').data(row);
                $row.enter().append('td');
                $row.text(String);
                $row.exit().remove();
            });
            $rows.exit().remove();
            _this.markReady();
        });
        return $table;
    };
    return Table;
}(AVisInstance));
export { Table };
export function create(data, parent, options) {
    return new Table(data, parent, options);
}
//# sourceMappingURL=table.js.map